-------------------------------------------------------------
--	Filename:  CLKGEN_PLL.VHD
-- Authors: 
--		Alain Zarembowitch / MSS, 
--	Version: 1
--	Date last modified: 3/5/11
-- Inheritance: 	COM-1600
--
-- description: Generate the key clocks (using a PLL) 
-- Generate the following clocks from the 20 MHz frequency reference: 
-- CLKOUT0G: 20 MHz  reference clock global
-- CLKOUT2G: 40 MHz I/O clock global
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
library unisim;
use unisim.vcomponents.all;

entity CLKGEN_PLL is
port
 (-- Clock in ports (directly from input pin)
  CLKREF_IN           : in     std_logic;
  -- Clock out ports (after BUFGs)
  CLKOUT0G          : out    std_logic;
  CLKOUT2G          : out    std_logic;
  -- Status and control signals
  LOCKED_PLL            : out    std_logic
 );
end entity;

architecture Behavioral of CLKGEN_PLL is
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLKREF_INB: std_logic;
signal CLKOUT0: std_logic;
signal CLKOUT2: std_logic;
signal CLKOUT0G_LOCAL: std_logic;

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin


  -- Input buffering
  --------------------------------------
  -- Global clock input buffer. The PLL will compensate the delay of this path (ug382)
  IBUFG_001: IBUFG port map
   (O => CLKREF_INB,
    I => CLKREF_IN);


  -- Clocking primitive
  --------------------------------------
  -- Instantiation of the PLL primitive
  --    * Unused inputs are tied off
  --    * Unused outputs are labeled unused

  pll_base_inst : PLL_BASE
  generic map
   (BANDWIDTH            => "OPTIMIZED",
    CLK_FEEDBACK         => "CLKOUT0",
    COMPENSATION         => "SOURCE_SYNCHRONOUS",
    DIVCLK_DIVIDE        => 1,	
    CLKFBOUT_MULT        => 1,	
    CLKFBOUT_PHASE       => 0.000,
    CLKOUT0_DIVIDE       => 30,	-- 20 MHz output. VCO 600 MHz [range 400 - 1000 MHz]
    CLKOUT0_PHASE        => 0.0,
    CLKOUT0_DUTY_CYCLE   => 0.500,
    CLKOUT2_DIVIDE       => 15,	-- CLK_IO 40 MHz 
    CLKOUT2_PHASE        => 0.0,
    CLKOUT2_DUTY_CYCLE   => 0.500,
    CLKIN_PERIOD         => 50.0,
    REF_JITTER           => 0.010)
  port map
    -- Output clocks
   (CLKFBOUT            => open,
    CLKOUT0             => CLKOUT0,
    CLKOUT1             => open,
    CLKOUT2             => CLKOUT2,
    CLKOUT3             => open,
    CLKOUT4             => open,
    CLKOUT5             => open,
    -- Status and control signals
    LOCKED              => LOCKED_PLL,
    RST                 => '0',
    -- Input clock control
    CLKFBIN             => CLKOUT0G_LOCAL,
    CLKIN               => CLKREF_INB);

  -- Output buffering
  -------------------------------------
BUFG_000: BUFG port map(I => CLKOUT0, O => CLKOUT0G_LOCAL);
BUFG_002: BUFG port map(I => CLKOUT2, O => CLKOUT2G);

CLKOUT0G <= CLKOUT0G_LOCAL;



end Behavioral;
